#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <sys/time.h>
#include "posix_time.h"
#include "clock_realtime.h"
#include "syscall_timer.h"


static int realtime_clock_gettime(clockid_t which_clock, struct timespec *ts)
{
  struct timeval tv;
  
  (void)which_clock;

  if (gettimeofday(&tv, NULL) < 0) return -1;
  ts->tv_sec = tv.tv_sec;
  ts->tv_nsec = tv.tv_usec * 1000;

  return 0;
}

static int realtime_clock_settime(clockid_t which_clock,
				  const struct timespec *setting)
{
  struct timeval tv;

  (void)which_clock;

  tv.tv_sec = setting->tv_sec;
  tv.tv_usec = setting->tv_nsec / 1000;
  return settimeofday(&tv, NULL);
}

static int realtime_clock_getres(clockid_t which_clock,
				 struct timespec *resolution)
{
  static int hz;
  static int errno_save;

  (void)which_clock;
  
  if (hz == 0) {
    errno = 0;
    hz = sysconf(_SC_CLK_TCK);
    errno_save = errno;
  }
  if (hz == -1) {
    errno = errno_save;
    return -1;
  }
  resolution->tv_sec = 0;
  resolution->tv_nsec = NSEC_PER_SEC/hz;

  return 0;
}

struct clock_func clock_realtime =
{
  realtime_clock_gettime,
  realtime_clock_settime,
  realtime_clock_getres,
  __timer_create,
  __timer_gettime,
  __timer_settime,
  __timer_getoverrun,
  __timer_delete
};
